# Generated by Django 4.2.13 on 2024-06-21 10:17

from django.db import migrations, models

import mozilla_django_oidc_db.fields


class Migration(migrations.Migration):

    dependencies = [
        (
            "digid_eherkenning_oidc_generics",
            "0006_alter_digidconfig_oidc_rp_scopes_list_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="digidconfig",
            name="default_loa",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
                        "DigiD Basis",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorContract",
                        "DigiD Midden",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:Smartcard",
                        "DigiD Substantieel",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:SmartcardPKI",
                        "DigiD Hoog",
                    ),
                ],
                help_text="Fallback level of assurance, in case no claim value could be extracted.",
                max_length=100,
                verbose_name="default LOA",
            ),
        ),
        migrations.AddField(
            model_name="digidconfig",
            name="loa_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                blank=True,
                default=None,
                help_text="Name of the claim holding the level of assurance. If left empty, it is assumed there is no LOA claim and the configured fallback value will be used.",
                null=True,
                size=None,
                verbose_name="LoA claim",
            ),
        ),
        migrations.AddField(
            model_name="digidmachtigenconfig",
            name="default_loa",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
                        "DigiD Basis",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:MobileTwoFactorContract",
                        "DigiD Midden",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:Smartcard",
                        "DigiD Substantieel",
                    ),
                    (
                        "urn:oasis:names:tc:SAML:2.0:ac:classes:SmartcardPKI",
                        "DigiD Hoog",
                    ),
                ],
                help_text="Fallback level of assurance, in case no claim value could be extracted.",
                max_length=100,
                verbose_name="default LOA",
            ),
        ),
        migrations.AddField(
            model_name="digidmachtigenconfig",
            name="loa_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                blank=True,
                default=None,
                help_text="Name of the claim holding the level of assurance. If left empty, it is assumed there is no LOA claim and the configured fallback value will be used.",
                null=True,
                size=None,
                verbose_name="LoA claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="default_loa",
            field=models.CharField(
                blank=True,
                choices=[
                    ("urn:etoegang:core:assurance-class:loa1", "Non existent (1)"),
                    ("urn:etoegang:core:assurance-class:loa2", "Low (2)"),
                    ("urn:etoegang:core:assurance-class:loa2plus", "Low (2+)"),
                    ("urn:etoegang:core:assurance-class:loa3", "Substantial (3)"),
                    ("urn:etoegang:core:assurance-class:loa4", "High (4)"),
                ],
                help_text="Fallback level of assurance, in case no claim value could be extracted.",
                max_length=100,
                verbose_name="default LOA",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="loa_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                blank=True,
                default=None,
                help_text="Name of the claim holding the level of assurance. If left empty, it is assumed there is no LOA claim and the configured fallback value will be used.",
                null=True,
                size=None,
                verbose_name="LoA claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningconfig",
            name="default_loa",
            field=models.CharField(
                blank=True,
                choices=[
                    ("urn:etoegang:core:assurance-class:loa1", "Non existent (1)"),
                    ("urn:etoegang:core:assurance-class:loa2", "Low (2)"),
                    ("urn:etoegang:core:assurance-class:loa2plus", "Low (2+)"),
                    ("urn:etoegang:core:assurance-class:loa3", "Substantial (3)"),
                    ("urn:etoegang:core:assurance-class:loa4", "High (4)"),
                ],
                help_text="Fallback level of assurance, in case no claim value could be extracted.",
                max_length=100,
                verbose_name="default LOA",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningconfig",
            name="loa_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                blank=True,
                default=None,
                help_text="Name of the claim holding the level of assurance. If left empty, it is assumed there is no LOA claim and the configured fallback value will be used.",
                null=True,
                size=None,
                verbose_name="LoA claim",
            ),
        ),
    ]
