# Generated by Django 4.2.13 on 2024-06-21 10:36

from django.db import migrations

import django_jsonform.models.fields


class Migration(migrations.Migration):

    dependencies = [
        (
            "digid_eherkenning_oidc_generics",
            "0007_digidconfig_default_loa_digidconfig_loa_claim_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="digidconfig",
            name="loa_value_mapping",
            field=django_jsonform.models.fields.JSONField(
                default=list,
                blank=True,
                help_text="Level of assurance claim value mappings. Useful if the values in the LOA claim are proprietary, so you can translate them into their standardized identifiers.",
                verbose_name="loa mapping",
            ),
        ),
        migrations.AddField(
            model_name="digidmachtigenconfig",
            name="loa_value_mapping",
            field=django_jsonform.models.fields.JSONField(
                default=list,
                blank=True,
                help_text="Level of assurance claim value mappings. Useful if the values in the LOA claim are proprietary, so you can translate them into their standardized identifiers.",
                verbose_name="loa mapping",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="loa_value_mapping",
            field=django_jsonform.models.fields.JSONField(
                default=list,
                blank=True,
                help_text="Level of assurance claim value mappings. Useful if the values in the LOA claim are proprietary, so you can translate them into their standardized identifiers.",
                verbose_name="loa mapping",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningconfig",
            name="loa_value_mapping",
            field=django_jsonform.models.fields.JSONField(
                default=list,
                blank=True,
                help_text="Level of assurance claim value mappings. Useful if the values in the LOA claim are proprietary, so you can translate them into their standardized identifiers.",
                verbose_name="loa mapping",
            ),
        ),
    ]
