# Generated by Django 4.2.13 on 2024-07-18 14:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("digid_eherkenning", "0009_decrypt_private_keys"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="digidconfiguration",
            name="key_passphrase",
        ),
        migrations.RemoveField(
            model_name="eherkenningconfiguration",
            name="key_passphrase",
        ),
    ]
