# Generated by Django 4.2.13 on 2024-05-24 16:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("digid_eherkenning_oidc_generics", "0002_auto_20240207_1546"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="OpenIDConnectPublicConfig",
            new_name="DigiDConfig",
        ),
        migrations.RenameModel(
            old_name="OpenIDConnectDigiDMachtigenConfig",
            new_name="DigiDMachtigenConfig",
        ),
        migrations.RenameModel(
            old_name="OpenIDConnectEHerkenningBewindvoeringConfig",
            new_name="EHerkenningBewindvoeringConfig",
        ),
        migrations.RenameModel(
            old_name="OpenIDConnectEHerkenningConfig",
            new_name="EHerkenningConfig",
        ),
    ]
