# Generated by Django 4.2.13 on 2024-06-12 06:50

from django.db import migrations, models

import django_jsonform.models.fields

import digid_eherkenning.oidc.models.base


class Migration(migrations.Migration):

    dependencies = [
        (
            "digid_eherkenning_oidc_generics",
            "0005_digidmachtigenconfig_mandate_service_id_claim_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="digidconfig",
            name="oidc_rp_scopes_list",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(
                    max_length=50, verbose_name="OpenID Connect scope"
                ),
                blank=True,
                default=digid_eherkenning.oidc.models.base.get_default_scopes_bsn,
                help_text="OpenID Connect scopes that are requested during login. These scopes are hardcoded and must be supported by the identity provider.",
                size=None,
                verbose_name="OpenID Connect scopes",
            ),
        ),
        migrations.AlterField(
            model_name="digidmachtigenconfig",
            name="oidc_rp_scopes_list",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(
                    max_length=50, verbose_name="OpenID Connect scope"
                ),
                blank=True,
                default=digid_eherkenning.oidc.models.base.get_default_scopes_bsn,
                help_text="OpenID Connect scopes that are requested during login. These scopes are hardcoded and must be supported by the identity provider.",
                size=None,
                verbose_name="OpenID Connect scopes",
            ),
        ),
        migrations.AlterField(
            model_name="eherkenningbewindvoeringconfig",
            name="oidc_rp_scopes_list",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(
                    max_length=50, verbose_name="OpenID Connect scope"
                ),
                blank=True,
                default=digid_eherkenning.oidc.models.base.get_default_scopes_bsn,
                help_text="OpenID Connect scopes that are requested during login. These scopes are hardcoded and must be supported by the identity provider.",
                size=None,
                verbose_name="OpenID Connect scopes",
            ),
        ),
        migrations.AlterField(
            model_name="eherkenningconfig",
            name="oidc_rp_scopes_list",
            field=django_jsonform.models.fields.ArrayField(
                base_field=models.CharField(
                    max_length=50, verbose_name="OpenID Connect scope"
                ),
                blank=True,
                default=digid_eherkenning.oidc.models.base.get_default_scopes_kvk,
                help_text="OpenID Connect scopes that are requested during login. These scopes are hardcoded and must be supported by the identity provider.",
                size=None,
                verbose_name="OpenID Connect scopes",
            ),
        ),
    ]
