# Generated by Django 4.2.13 on 2024-06-11 20:44

from django.db import migrations, models

import mozilla_django_oidc_db.fields


class Migration(migrations.Migration):

    dependencies = [
        (
            "digid_eherkenning_oidc_generics",
            "0004_migrate_config_to_claimfield",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="digidmachtigenconfig",
            name="mandate_service_id_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                default=mozilla_django_oidc_db.fields.ClaimFieldDefault(
                    "urn:nl-eid-gdi:1.0:ServiceUUID"
                ),
                help_text="Name of the claim holding the service UUID for which the acting subject is authorized.",
                size=None,
                verbose_name="service ID claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="branch_number_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                default=mozilla_django_oidc_db.fields.ClaimFieldDefault(
                    "urn:etoegang:1.9:ServiceRestriction:Vestigingsnr"
                ),
                help_text="Name of the claim holding the value of the branch number for the authenticated company, if such a restriction applies.",
                size=None,
                verbose_name="branch number claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="mandate_service_id_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                default=mozilla_django_oidc_db.fields.ClaimFieldDefault(
                    "urn:etoegang:core:ServiceID"
                ),
                help_text="Name of the claim holding the service ID for which the company is authorized.",
                size=None,
                verbose_name="service ID claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningbewindvoeringconfig",
            name="mandate_service_uuid_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                default=mozilla_django_oidc_db.fields.ClaimFieldDefault(
                    "urn:etoegang:core:ServiceUUID"
                ),
                help_text="Name of the claim holding the service UUID for which the company is authorized.",
                size=None,
                verbose_name="service UUID claim",
            ),
        ),
        migrations.AddField(
            model_name="eherkenningconfig",
            name="branch_number_claim",
            field=mozilla_django_oidc_db.fields.ClaimField(
                base_field=models.CharField(
                    max_length=50, verbose_name="claim path segment"
                ),
                default=mozilla_django_oidc_db.fields.ClaimFieldDefault(
                    "urn:etoegang:1.9:ServiceRestriction:Vestigingsnr"
                ),
                help_text="Name of the claim holding the value of the branch number for the authenticated company, if such a restriction applies.",
                size=None,
                verbose_name="branch number claim",
            ),
        ),
    ]
